import * as GC from "@grapecity/spread-sheets"
import { flagtestJSON } from "./assests/sampleJSON";
import { largeFileTemplate } from "./assests/incrmentalJSON";
import { circularDependencyJSON } from "./assests/circularDependencyJSON";
import { withArrayFormulaTemplate, withoutArrayFormulaTemplate } from "./assests/ArrayFormulaTemplates";
import { volatileCondionTempalte, nonVolatileCondionTempalte } from "./assests/condionalFormatting";
export function testPainting(spread: GC.Spread.Sheets.Workbook, isSuspend: boolean) {
    let sheet = spread.getActiveSheet();
    let setDummyData = (sheet: GC.Spread.Sheets.Worksheet, rowCount: number, colCount: number) => {
        for (let i = 0; i < rowCount; i++) {
            for (let j = 0; j < colCount; j++) {
                sheet.setValue(i, j, "text( " + i
                    + ", " + j + ")")
            }
        }
    }
    console.time("test1");
    isSuspend && sheet.suspendPaint()
    const start = performance.now(); // returns something like 138.899999998509884, which means 138.9 milliseconds passed
    setDummyData(sheet, 10, 10)
    const duration = performance.now() - start;
    isSuspend && sheet.resumePaint();

    updateValue(isSuspend, duration)

    return duration

}

export function testCalculation(spread: GC.Spread.Sheets.Workbook, isSuspend: boolean) {
    let sheet = spread.getActiveSheet();

    let setDummyData = (sheet: GC.Spread.Sheets.Worksheet, rowCount: number, colCount: number) => {
        for (let i = 0; i < rowCount; i++) {
            for (let j = 0; j < colCount; j++) {
                sheet.setFormula(i, j, "=Sum(1,2,3)")
            }
        }
    }
    console.time("test1");
    sheet.suspendPaint()
    isSuspend && sheet.suspendCalcService()
    const start = performance.now(); // returns something like 138.899999998509884, which means 138.9 milliseconds passed
    setDummyData(sheet, 10, 10)
    const duration = performance.now() - start;
    isSuspend && sheet.resumeCalcService();
    sheet.resumePaint();
    updateValue(isSuspend, duration)
    return duration

}

export function testFlags(spread: GC.Spread.Sheets.Workbook, flags: any) {
    if (flags) {
        flags.forEach((flag: any) => {
            (spread.options as any)[flag] = true
        });
    }
    const start = performance.now(); // returns something like 138.899999998509884, which means 138.9 milliseconds passed
    spread.fromJSON(flagtestJSON, flags ? { doNotRecalculateAfterLoad: true } : null)
    const duration = performance.now() - start;
    updateValue(flags ? true : false, duration)
    return duration

}

export function testIncrementlLaoding(spread: GC.Spread.Sheets.Workbook, isIncrement: boolean) {


    let isUpdated = false
    const start = performance.now(); // returns something like 138.899999998509884, which means 138.9 milliseconds passed
    var loadStats = {
        loading: function (progress: number, args: any) {
            // The loading progress from 0 to 1.
            // args.sheet the sheet currently loading  
            if (!isUpdated) {
                updateValue(true, "First chunk loaded in " + (performance.now() - start) + "ms");
                isUpdated = true
            }

        },
        loaded: function () {
            // The works to do when loading finish.
        }
    }
    spread.fromJSON(largeFileTemplate, { incrementalLoading: isIncrement ? loadStats : false })

    if (!isIncrement) updateValue(false, "Whole file loaded in " + (performance.now() - start) + "ms")

}

export function testItrativeCal(spread: GC.Spread.Sheets.Workbook, showChanges: boolean) {
    spread.options.iterativeCalculation = true;
    spread.options.iterativeCalculationMaximumIterations = showChanges ? 2 : 10000;
    spread.options.iterativeCalculationMaximumChange = 0.1;
    spread.getActiveSheet().reset()
    const start = performance.now();
    spread.getActiveSheet().fromJSON(circularDependencyJSON);
    updateValue(showChanges, "the json laoded in " + (performance.now() - start) + "ms")
}

export function testNamedStyle(spread: GC.Spread.Sheets.Workbook, useNamedStyle: boolean) {
    let sheet = spread.getActiveSheet();
    sheet.reset();

    let setStyleAllColumns = (style: any) => {
        sheet.suspendPaint()
        for (let i = 0; i < sheet.getRowCount(); i++) {
            for (let j = 0; j < sheet.getColumnCount(); j++) {
                if (typeof style === "string") {
                    sheet.setStyleName(i, j, style)
                } else {
                    sheet.setStyle(i, j, style)
                }
            }
        }
        sheet.resumePaint()
    }
    let start = performance.now()
    if (useNamedStyle) {
        let style = new GC.Spread.Sheets.Style()
        style.backColor = "green";
        style.foreColor = "red";
        style.name = "commonStyle";
        spread.addNamedStyle(style);
        setStyleAllColumns("commonStyle")
    } else {
        let style = new GC.Spread.Sheets.Style()
        style.backColor = "green";
        style.foreColor = "red";
        setStyleAllColumns(style)
    }
    const size = new TextEncoder().encode(JSON.stringify(spread.toJSON())).length
    const kiloBytes = size / 1024;
    updateValue(useNamedStyle, "The Spread Size is" + kiloBytes + "kb")
}

export function testStaticFormulas(spread: GC.Spread.Sheets.Workbook, useFormula: boolean) {
    let sheet = spread.getActiveSheet();
    sheet.reset();
    let setFormulaToSheet = (isValue: any) => {
        sheet.suspendPaint()
        for (let i = 0; i < sheet.getRowCount(); i++) {
            for (let j = 0; j < sheet.getColumnCount(); j++) {
                if (!isValue) {
                    sheet.setFormula(i, j, "=" + i)
                } else {
                    sheet.setValue(i, j, i)
                }
            }
        }
        sheet.resumePaint()
    }
    let start = performance.now()
    setFormulaToSheet(useFormula ? false : true);
    let params = useFormula ? "Formuals" : "values"
    updateValue(useFormula, "Setting " + params + " takes" + (performance.now() - start) + "ms")
}

export function testArrayFormula(spread: GC.Spread.Sheets.Workbook, loadArrayFormulaSheet: boolean) {
    spread = reInitialize(spread)
    let sheet = spread.getActiveSheet()
    spread.options.allowDynamicArray = loadArrayFormulaSheet;

    let template = loadArrayFormulaSheet ? withArrayFormulaTemplate : withoutArrayFormulaTemplate
    console.log("loading template")

    sheet.suspendPaint();
    sheet.suspendCalcService();
    (sheet as any).fromJSON(template, { doNotRecalculateAfterReload: true });
    sheet.getRange(1, 1, 100, 30).value(null);
    sheet.resumeCalcService();
    sheet.resumePaint();
    const start = performance.now();
    sheet.recalcAll()
    sheet.repaint()



    updateValue(loadArrayFormulaSheet, "the sheet laoded in " + (performance.now() - start) + "ms")
}

export function testCondionalFormatting(spread: GC.Spread.Sheets.Workbook, isVolatile: any) {

    const start = performance.now();
    spread.suspendPaint();
    spread.fromJSON(isVolatile ? volatileCondionTempalte : nonVolatileCondionTempalte)
    spread.resumePaint()
    const duration = performance.now() - start;
    updateValue(isVolatile ? true : false, "File loaded in " + duration + "ms");

}


function updateValue(isWithMethod: boolean, value: any) {

    let className = isWithMethod ? ".with" : ".without";
    document.querySelector(".grid-item" + className + ".value").textContent = value
}

function reInitialize(spread: GC.Spread.Sheets.Workbook) {
    spread.destroy()
    spread = new GC.Spread.Sheets.Workbook(document.getElementById("ss"), { sheetCount: 1 });
    return spread;
}